@@ -20,7 +20,7 @@ except ImportError:
 
 
 
-host_pattern = r'((?P<user>[^@]+)@)?(?P<host>[^:]+)(:(?P<port>\d+))?'
+host_pattern = r'((?P<user>.+)@)?(?P<host>[^:]+)(:(?P<port>\d+))?'
 host_regex = re.compile(host_pattern)
 
 
@@ -285,7 +285,6 @@ def prompt_for_password(previous=None, prompt=None):
     # returning the empty string, and to avoid unnecessary network overhead.)
     while not new_password:
         print("Sorry, you can't enter an empty password. Please try again.")
-        password_prompt = base_password_prompt + ": "
         new_password = getpass.getpass(password_prompt)
     return new_password
 
